/*
 * jSite2 - Core.java -
 * Copyright © 2008 David Roden
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package net.pterodactylus.jsite.core;

import java.io.IOException;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.List;

import net.pterodactylus.jsite.project.Project;
import net.pterodactylus.jsite.project.ProjectManager;

/**
 * The core of jSite.
 * 
 * @author David ‘Bombe’ Roden &lt;bombe@freenetproject.org&gt;
 */
public class CoreImpl implements Core, NodeListener, RequestListener {

	/** The core listeners. */
	private final List<CoreListener> coreListeners = new ArrayList<CoreListener>();

	/** The project manager. */
	private ProjectManager projectManager;

	/** The node manager. */
	private NodeManager nodeManager;

	/** The request manager. */
	/* TODO - remove */
	@SuppressWarnings("unused")
	private RequestManager requestManager;

	//
	// LISTENER MANAGEMENT
	//

	/**
	 * {@inheritDoc}
	 */
	public void addCoreListener(CoreListener coreListener) {
		coreListeners.add(coreListener);
	}

	/**
	 * {@inheritDoc}
	 */
	public void removeCoreListener(CoreListener coreListener) {
		coreListeners.remove(coreListener);
	}

	/**
	 * Notifies all listeners that the projects were loaded successfully.
	 * 
	 * @param directory
	 *            The directory the projects were loaded from
	 */
	private void fireLoadingProjectsDone(String directory) {
		for (CoreListener coreListener: coreListeners) {
			coreListener.loadingProjectsDone(directory);
		}
	}

	/**
	 * Notifies all core listeners that loading the projects from the given
	 * directory has failed.
	 * 
	 * @param directory
	 *            The directory the projects were tried to load from
	 * @param throwable
	 *            The exception that occured when loading projects
	 */
	private void fireLoadingProjectsFailed(String directory, Throwable throwable) {
		for (CoreListener coreListener: coreListeners) {
			coreListener.loadingProjectsFailed(directory, throwable);
		}
	}

	/**
	 * Notifies all listeners that the projects were successfully saved.
	 * 
	 * @param directory
	 *            The directory the projects were saved to
	 */
	private void fireSavingProjectsDone(String directory) {
		for (CoreListener coreListener: coreListeners) {
			coreListener.savingProjectsDone(directory);
		}
	}

	/**
	 * Notifies all listeners that the projects could not be saved.
	 * 
	 * @param directory
	 *            The directory the projects were to be saved to
	 * @param throwable
	 *            The exception that occured when saving the projects
	 */
	private void fireSavingProjectsFailed(String directory, Throwable throwable) {
		for (CoreListener coreListener: coreListeners) {
			coreListener.savingProjectsFailed(directory, throwable);
		}
	}

	/**
	 * Notifies all listeners that a project was added.
	 * 
	 * @param project
	 *            The project that was added
	 */
	private void fireProjectAdded(Project project) {
		for (CoreListener coreListener: coreListeners) {
			coreListener.projectAdded(project);
		}
	}
	
	/**
	 * Notifies all listeners that a project was cloned.
	 * @param clonedProject The project that was cloned
	 * @param projectClone The clone of the project
	 */
	private void fireProjectCloned(Project clonedProject, Project projectClone) {
		for (CoreListener coreListener: coreListeners) {
			coreListener.projectCloned(clonedProject, projectClone);
		}
	}

	/**
	 * Notifies all listeners that a project was removed.
	 * 
	 * @param project
	 *            The project that was removed
	 */
	private void fireProjectRemoved(Project project) {
		for (CoreListener coreListener: coreListeners) {
			coreListener.projectRemoved(project);
		}
	}

	/**
	 * Notifies all listeners that the nodes were successfully loaded.
	 * 
	 * @param directory
	 *            The directory the nodes were loaded from
	 */
	private void fireLoadingNodesDone(String directory) {
		for (CoreListener coreListener: coreListeners) {
			coreListener.loadingNodesDone(directory);
		}
	}

	/**
	 * Notifies all listeners that loading the nodes has failed.
	 * 
	 * @param directory
	 *            The directory the nodes were loaded from
	 * @param throwable
	 *            The exception that occured while loading the nodes
	 */
	private void fireLoadingNodesFailed(String directory, Throwable throwable) {
		for (CoreListener coreListener: coreListeners) {
			coreListener.loadingNodesFailed(directory, throwable);
		}
	}

	/**
	 * Notifies all listeners that the nodes were saved successfully.
	 * 
	 * @param directory
	 *            The directory the nodes were saved to
	 */
	private void fireSavingNodesDone(String directory) {
		for (CoreListener coreListener: coreListeners) {
			coreListener.savingNodesDone(directory);
		}
	}

	/**
	 * Notifies all listeners that saving the nodes has failed.
	 * 
	 * @param directory
	 *            The directory the nodes were saved to
	 * @param throwable
	 *            The exception that occured while saving the nodes
	 */
	private void fireSavingNodesFailed(String directory, Throwable throwable) {
		for (CoreListener coreListener: coreListeners) {
			coreListener.savingProjectsFailed(directory, throwable);
		}
	}

	/**
	 * Notifies all core listeners that the core has loaded and is ready to run.
	 */
	private void fireCoreLoaded() {
		for (CoreListener coreListener: coreListeners) {
			coreListener.coreLoaded();
		}
	}

	/**
	 * Notifies all listeners that the core was stopped.
	 */
	private void fireCoreStopped() {
		for (CoreListener coreListener: coreListeners) {
			coreListener.coreStopped();
		}
	}

	/**
	 * Notifies all listeners that a node was added to the core.
	 * 
	 * @param node
	 *            The node that was added
	 */
	private void fireNodeAdded(Node node) {
		for (CoreListener coreListener: coreListeners) {
			coreListener.nodeAdded(node);
		}
	}

	/**
	 * Notifies all listeners that a node was removed from the core.
	 * 
	 * @param node
	 *            The node that was removed
	 */
	private void fireNodeRemoved(Node node) {
		for (CoreListener coreListener: coreListeners) {
			coreListener.nodeRemoved(node);
		}
	}

	/**
	 * Notifies all listeners that a connection to the given node is now being
	 * established.
	 * 
	 * @param node
	 *            The node that is being connected to
	 */
	private void fireNodeConnecting(Node node) {
		for (CoreListener coreListener: coreListeners) {
			coreListener.nodeConnecting(node);
		}
	}

	/**
	 * Notifies all listeners that the given node is now connected.
	 * 
	 * @param node
	 *            The node that is now connected
	 */
	private void fireNodeConnected(Node node) {
		for (CoreListener coreListener: coreListeners) {
			coreListener.nodeConnected(node);
		}
	}

	/**
	 * Notifies all listeners that a connection to a node has failed.
	 * 
	 * @param node
	 *            The node that could not be connected
	 * @param cause
	 *            The cause of the failure
	 */
	private void fireNodeConnectionFailed(Node node, Throwable cause) {
		for (CoreListener coreListener: coreListeners) {
			coreListener.nodeConnectionFailed(node, cause);
		}
	}

	/**
	 * Notifies all listeners that the given node was disconnected.
	 * 
	 * @param node
	 *            The node that is now disconnected
	 * @param throwable
	 *            The exception that caused the disconnect, or <code>null</code>
	 *            if there was no exception
	 */
	private void fireNodeDisconnected(Node node, Throwable throwable) {
		for (CoreListener coreListener: coreListeners) {
			coreListener.nodeDisconnected(node, throwable);
		}
	}

	//
	// ACCESSORS
	//

	/**
	 * Returns the project manager.
	 * 
	 * @return The project manager
	 */
	public ProjectManager getProjectManager() {
		return projectManager;
	}

	/**
	 * Sets the project manager to use.
	 * 
	 * @param projectManager
	 *            The project manager to use
	 */
	public void setProjectManager(ProjectManager projectManager) {
		this.projectManager = projectManager;
	}

	/**
	 * Returns the node manager.
	 * 
	 * @return The node manager
	 */
	public NodeManager getNodeManager() {
		return nodeManager;
	}

	/**
	 * Sets the node manager to use.
	 * 
	 * @param nodeManager
	 *            The node manager to use
	 */
	public void setNodeManager(NodeManager nodeManager) {
		this.nodeManager = nodeManager;
	}

	/**
	 * Sets the request manager to use.
	 * 
	 * @param requestManager
	 *            The request manager to use
	 */
	public void setRequestManager(RequestManager requestManager) {
		this.requestManager = requestManager;
	}

	/**
	 * {@inheritDoc}
	 */
	public List<Node> getNodes() {
		return nodeManager.getNodes();
	}

	/**
	 * {@inheritDoc}
	 */
	public boolean isNodeConnected(Node node) {
		return nodeManager.hasNode(node);
	}

	/**
	 * {@inheritDoc}
	 */
	public List<Project> getProjects() {
		return projectManager.getProjects();
	}

	//
	// ACTIONS
	//

	/**
	 * {@inheritDoc}
	 */
	public void start() {
		try {
			projectManager.load();
			fireLoadingProjectsDone(projectManager.getDirectory());
		} catch (IOException ioe1) {
			fireLoadingProjectsFailed(projectManager.getDirectory(), ioe1);
		}
		try {
			nodeManager.load();
			fireLoadingNodesDone(nodeManager.getDirectory());
		} catch (IOException ioe1) {
			fireLoadingNodesFailed(nodeManager.getDirectory(), ioe1);
		}
		fireCoreLoaded();
	}

	/**
	 * {@inheritDoc}
	 */
	public void stop() {
		try {
			projectManager.save();
			fireSavingProjectsDone(projectManager.getDirectory());
		} catch (IOException ioe1) {
			fireSavingProjectsFailed(projectManager.getDirectory(), ioe1);
		}
		try {
			nodeManager.save();
			fireSavingNodesDone(nodeManager.getDirectory());
		} catch (IOException ioe1) {
			fireSavingNodesFailed(nodeManager.getDirectory(), ioe1);
		}
		fireCoreStopped();
	}

	/**
	 * {@inheritDoc}
	 */
	public boolean addNode(Node node) throws UnknownHostException {
		return nodeManager.addNode(node);
	}

	/**
	 * {@inheritDoc}
	 */
	public void removeNode(Node node) {
		nodeManager.removeNode(node);
	}

	/**
	 * {@inheritDoc}
	 */
	public void connectToNode(Node node) {
		fireNodeConnecting(node);
		nodeManager.connect(node);
	}

	/**
	 * {@inheritDoc}
	 */
	public void disconnectFromNode(Node node) {
		nodeManager.disconnect(node);
	}

	/**
	 * {@inheritDoc}
	 */
	public void createProject() throws IOException, JSiteException {
		Project newProject = projectManager.createProject();
		fireProjectAdded(newProject);
	}

	/**
	 * {@inheritDoc}
	 */
	public void cloneProject(Project project) {
		Project projectClone = projectManager.cloneProject(project);
		fireProjectCloned(project, projectClone);
	}
	
	/**
	 * {@inheritDoc}
	 */
	public void removeProject(Project project) {
		projectManager.removeProject(project);
		fireProjectRemoved(project);
	}
	
	//
	// PRIVATE METHODS
	//

	/**
	 * Loads the configuration.
	 */
	@SuppressWarnings("unused")
	private void loadConfig() {
		/* TODO */
	}

	/**
	 * Saves the configuration.
	 */
	@SuppressWarnings("unused")
	private void saveConfig() {
		/* TODO */
	}

	//
	// INTERFACE NodeListener
	//

	/**
	 * {@inheritDoc}
	 */
	public void nodeAdded(Node node) {
		fireNodeAdded(node);
	}

	/**
	 * {@inheritDoc}
	 */
	public void nodeRemoved(Node node) {
		fireNodeRemoved(node);
	}

	/**
	 * {@inheritDoc}
	 */
	public void nodeConnected(Node node) {
		fireNodeConnected(node);
	}

	/**
	 * {@inheritDoc}
	 */
	public void nodeConnectionFailed(Node node, Throwable cause) {
		fireNodeConnectionFailed(node, cause);
	}

	/**
	 * {@inheritDoc}
	 */
	public void nodeDisconnected(Node node, Throwable throwable) {
		fireNodeDisconnected(node, throwable);
	}

	//
	// INTERFACE RequestListener
	//

	/**
	 * {@inheritDoc}
	 */
	public void requestAdded(Request request) {
		/* TODO - find project and process request */
	}

	/**
	 * @see net.pterodactylus.jsite.core.RequestListener#requestProgressed(Request)
	 */
	public void requestProgressed(Request request) {
		/* TODO - find project and process request */
	}

	/**
	 * @see net.pterodactylus.jsite.core.RequestListener#requestRemoved(net.pterodactylus.jsite.core.Request)
	 */
	public void requestRemoved(Request request) {
		/* TODO - find project and process request */
	}

	/**
	 * @see net.pterodactylus.jsite.core.RequestListener#requestGeneratedURI(net.pterodactylus.jsite.core.Request,
	 *      java.lang.String)
	 */
	public void requestGeneratedURI(Request request, String uri) {
		/* TODO - find project and process request */
	}

	/**
	 * @see net.pterodactylus.jsite.core.RequestListener#requestFinished(net.pterodactylus.jsite.core.Request)
	 */
	public void requestFinished(Request request) {
		/* TODO - find project and process request */
	}

}
